package com.biganto.visual.roompark.presentation.screen.albums

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.AlbumsInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class AlbumsScreenPresenter @Inject constructor(
    private val interactor: AlbumsInteractor
    ,@Named(SELECTED_ALBUM_INDEX_KEY) private var selectedIndex:Int
)
    : BigantoBasePresenter<AlbumsScreen, AlbumsScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> AlbumsScreenViewState.SomeError(e) }

    private fun requestAlbum(id:Int):Observable<AlbumsScreenViewState> =
        interactor.fetchAlbumPhotos(id)
        .filter { !it.isNullOrEmpty() }
        .map { AlbumsScreenViewState.AlbumsSelected(it)}

    override fun bindIntents() {

        val fetchParents = interactor.fetchHeaderAlbums()
            .filter { !it.isNullOrEmpty() }
            .map { AlbumsScreenViewState.AlbumsListLoaded(it, selectedIndex) }

        val fetchSelected = requestAlbum(selectedIndex)


        val headerItemSelected = intent(AlbumsScreen::onAlbumSelected)
            .doOnNext { selectedIndex = it.albumId }
            .flatMap<AlbumsScreenViewState> { model ->
                requestAlbum(model.albumId)
                    .startWith(Observable.just(AlbumsScreenViewState.HeaderAlbumChoosed(item = model)))
            }

        val photoSelected = intent(AlbumsScreen::onPhotoSelected)
            .map { AlbumsScreenViewState.PhotoSelected(it.photoId) }




        val state = restoreStateObservable
            .mergeWith(fetchParents)
            .mergeWith(fetchSelected)
            .mergeWith(headerItemSelected)
            .mergeWith(photoSelected)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
        subscribeViewState(state.cast(AlbumsScreenViewState::class.java), AlbumsScreen::render)
    }

}