package com.biganto.visual.roompark.presentation.screen.article

import android.os.Bundle
import android.text.Html
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.os.bundleOf
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import butterknife.OnClick
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.article.util.HtmlPageAdapter
import com.biganto.visual.roompark.presentation.screen.article.util.HtmlTag
import com.biganto.visual.roompark.util.extensions.formatToSimple
import com.google.android.material.floatingactionbutton.FloatingActionButton
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import jp.wasabeef.picasso.transformations.BlurTransformation
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val ARTICLE_ID = "DIRECT_ARTICLE_ID_KEY"

class ArticleScreenController :
    BigantoBaseController<ArticleScreenViewState
            , ArticleScreen
            , ArticleScreenPresenter>
    , ArticleScreen {

    constructor(args:Bundle):super(args)

    constructor(id: Int) : super(bundleOf(ARTICLE_ID to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: ArticleScreenPresenter

    @BindView(R.id.articleTitle)
    lateinit var title:MaterialTextView

    @BindView(R.id.articleHeaderBlock)
    lateinit var headerBlock: ViewGroup


    @BindView(R.id.articleCloseButton)
    lateinit var closeArticle: FloatingActionButton

    @BindView(R.id.articleBodyRecyclerView)
    lateinit var articleRecyclerView: RecyclerView



    val blurPreview:ImageView by lazy {
       headerBlock.findViewById<ImageView>(R.id.articlePreviewBlurred)
    }

    val articlePreview:ImageView by lazy {
        headerBlock.findViewById<ImageView>(R.id.articlePreview)
    }

    val articleDate:MaterialTextView by lazy {
        headerBlock.findViewById<MaterialTextView>(R.id.articleDate)
    }

    val closeButton:FloatingActionButton by lazy {
        headerBlock.findViewById<FloatingActionButton>(R.id.articleCloseButton)
    }

    @OnClick(R.id.articleCloseButton)
    fun onCloseArticle(){
        handleBack()
    }

    private fun bindRecycler() {
        articleRecyclerView.isNestedScrollingEnabled = true
        articleRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        articleRecyclerView.adapter = HtmlPageAdapter()
        articleRecyclerView.itemAnimator = null
    }

    override fun onViewBound(v: View) {
        toolBar.setToolbar()
        bottomNavigationController.hide()
        bindRecycler()
    }

    override fun render(viewState: ArticleScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticleScreenViewState.Idle -> render(viewState)
            is ArticleScreenViewState.ArticleLoaded -> render(viewState)
        }
    }

    private fun render(viewState: ArticleScreenViewState.Idle){

    }


    private fun render(viewState: ArticleScreenViewState.ArticleLoaded) {

        title.text = viewState.item.title

//        val imageGetter = PicassoImageGetter()

        val _escaped = Html.escapeHtml(viewState.item.htmlBody)
        val charset = Charsets.UTF_8

        val reg = "<img\\b(?=\\s)(?=(?:[^>=]|='[^']*'|=\"[^\"]*\"|=[^'\"][^\\s>]*)*?\\ssrc=['\"]([^\"]*)['\"]?)(?:[^>=]|='[^']*'|=\"[^\"]*\"|=[^'\"\\s]*)*\"\\s?\\/?>"
            .toRegex()

        val _esc =_escaped.toByteArray(charset).contentToString()
        val z = viewState.item.htmlBody.replace("<\\br>","\n")
            .replace("<br>","\n")
            .replace("</br>","\n").toString()

        Timber.d(" escaped lines: ${z}")

        val tags = z.lines().map{
            val r = reg.matchEntire(it)?.groupValues?.last()
            Timber.d("matched: $r")
            val result = if (r == null) HtmlTag.Text(it)
                        else HtmlTag.ImageSource(r)
            Timber.d(" result: ${result}")
            result

        }.toList()

        (articleRecyclerView.adapter as HtmlPageAdapter).setItems(tags)

        articleDate.text = viewState.item.published.formatToSimple

        Picasso.get()
            .load(viewState.item.previewUrl)
            .transform(BlurTransformation(activity,40,4))
            .into(blurPreview)

        Picasso.get()
            .load(viewState.item.previewUrl)
            .into(articlePreview)
    }

    private fun getComponent() = DaggerArticleScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(ARTICLE_ID))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_read_screen


}

