package com.biganto.visual.roompark.presentation.screen.article.util

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import timber.log.Timber

/**
 * Created by Vladislav Bogdashkin on 29.01.2020.
 */



sealed class HtmlTag {
    class Text(val text:String):HtmlTag()
    class ImageSource(val src:String):HtmlTag()
}



class HtmlPageAdapter : RecyclerView.Adapter<HtmlTagViewHolder<HtmlTag>>() {


    private val list = mutableListOf<HtmlTag>()

    override fun getItemViewType(position: Int): Int {
        return when (list[position]){
            is HtmlTag.Text -> HtmlViewType.TEXT.viewType
            is HtmlTag.ImageSource -> HtmlViewType.IMG.viewType
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): HtmlTagViewHolder<HtmlTag> {
      val ret = when (HtmlViewType.fromInt(viewType)) {
            HtmlViewType.TEXT ->
                HtmlTextViewHolder(
                    LayoutInflater.from(parent.context)
                        .inflate(R.layout.htlm_text_viewholder, parent, false)
                ) as HtmlTagViewHolder<HtmlTag>
            HtmlViewType.IMG ->
                HtmlImageViewHolder(
                    LayoutInflater.from(parent.context)
                        .inflate(R.layout.htlm_image_viewholder, parent, false)
                ) as HtmlTagViewHolder<HtmlTag>
        }

        return  ret
    }


    override fun onBindViewHolder(holder: HtmlTagViewHolder<HtmlTag>, position: Int) {
        Timber.d(list[position].toString())
        Timber.d(" holder: ${holder::class} - list: ${list[position]::class}")
        Timber.d("position - $position")
        holder.bindModel(list[position])
    }

    override fun getItemCount(): Int = list.size


    fun setItems(items:List<HtmlTag>){
        this.list.clear()
        this.list.addAll(items)
        notifyDataSetChanged()
    }
}

abstract class HtmlTagViewHolder<M:HtmlTag>(itemView: View) : RecyclerView.ViewHolder(itemView){

    abstract fun onViewBound(model: M)

    fun bindModel(model: M){
        ButterKnife.bind(this, itemView)
        bindedModel = model
        onViewBound(model)
    }


    protected lateinit var bindedModel: M
}

class HtmlTextViewHolder(itemView: View) : HtmlTagViewHolder<HtmlTag.Text>(itemView) {

    override fun onViewBound(model: HtmlTag.Text) {
        (itemView as MaterialTextView).text = model.text
    }

}

class HtmlImageViewHolder(itemView: View) :HtmlTagViewHolder<HtmlTag.ImageSource>(itemView) {

    override fun onViewBound(model: HtmlTag.ImageSource) {
        Picasso.get().load(
            "https://room-park.ru${model.src}"
        )
            .into(itemView as ImageView)
    }

}
private enum class HtmlViewType(val viewType:Int){

    TEXT(0),
    IMG(1);

    companion object {
        fun fromInt(type: Int) = HtmlViewType.values().first { it.viewType == type }
    }


}
