package com.biganto.visual.roompark.presentation.screen.photo

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.PhotoInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class PhotoScreenPresenter @Inject constructor(
    private val interactor: PhotoInteractor
    ,@Named(SELECTED_PHOTO_KEY) private var selectedIndex:Int
)
    : BigantoBasePresenter<PhotoScreen, PhotoScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> PhotoScreenViewState.SomeError(e) }

    override fun bindIntents() {

        val fetchAlbum = interactor.fetchPhoto(selectedIndex)
            .flatMap { photo ->
                Observable.merge<PhotoScreenViewState>(
                    arrayListOf(
                        interactor.getAlbum(photo.albumId)
                            .map { PhotoScreenViewState.AldumFetched(it) }
                        ,
                        interactor.fetchPhotos(photo.albumId)
                            .map<PhotoScreenViewState> {
                                PhotoScreenViewState.PhotoListLoaded(it,selectedIndex)
                            }
                            .startWith(Observable.just(PhotoScreenViewState.PhotoFetched(photo)))
                    ))
            }

        val state = restoreStateObservable
            .mergeWith(fetchAlbum)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
        subscribeViewState(state.cast(PhotoScreenViewState::class.java), PhotoScreen::render)
    }
}