package com.biganto.visual.roompark.presentation.screen.photo.util

import android.view.View
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.domain.model.PhotoModel
import com.biganto.visual.roompark.domain.model.PhotoResolutionModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.biganto.visual.roompark.util.view_utils.image_view.RoundedImageView

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */


class PhotosAdapter : CommonRecyclerAdapter<PhotosViewHolder, PhotoModel>() {
    override val vhKlazz = PhotosViewHolder::class

    override fun getVhLayout(): Int = R.layout.photo_preview_viewholder
}

class PhotosViewHolder(itemView: View) : CommonViewHolder<PhotoModel>(itemView) {

    @BindView(R.id.photo_preview_imageview) lateinit var photoPreview: RoundedImageView

    private val picassoAsync by lazy {
        return@lazy RoomParkApplication.component.providePicassoAsync()
    }

    override fun onViewBound(model: PhotoModel) {
        model.resolutionList.lowelest()?.let {
            picassoAsync
                .load(it.url)
                .centerCrop()
                .fit()
                .into(photoPreview)
        }}
}

fun List<PhotoResolutionModel>.lowelest() =
    this.minBy { it.resWidth * it.resHeight }