package com.biganto.visual.roompark.presentation.screen.photo.util

import android.app.Activity
import android.graphics.Bitmap
import android.graphics.Point
import android.graphics.drawable.Drawable
import android.util.DisplayMetrics
import android.view.View
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.domain.model.PhotoModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.github.chrisbanes.photoview.PhotoView
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Callback
import com.squareup.picasso.Picasso
import com.squareup.picasso.Target
import jp.wasabeef.picasso.transformations.BlurTransformation
import jp.wasabeef.picasso.transformations.ColorFilterTransformation
import timber.log.Timber


/**
 * Created by Vladislav Bogdashkin on 15.10.2019.
 */
class PhotoViewerAdapter : CommonRecyclerAdapter<PhotoViewerViewHolder, PhotoModel>() {

    override val vhKlazz = PhotoViewerViewHolder::class

    override fun getVhLayout(): Int = R.layout.photo_page_viewholder

    fun getItemPosition(model: PhotoModel) = getItemPosition(model.photoId)

    fun getItemPosition(modelId: Int) = list.indexOfFirst {
        it.photoId == modelId
    }
}


class PhotoViewerViewHolder(itemView: View) : CommonViewHolder<PhotoModel>(itemView) {

    @BindView(R.id.photo_view) lateinit var photoView:PhotoView
    @BindView(R.id.photo_description) lateinit var photoDesc:MaterialTextView

    private val displayMetrics = DisplayMetrics()
    private val displaySize = Point()

    private val picassoAsync: Picasso by lazy {
        return@lazy RoomParkApplication.component.providePicassoAsync()
    }

    private val _t = object :Target{
        override fun onPrepareLoad(placeHolderDrawable: Drawable?) {
            Timber.d("lalala")
        }

        override fun onBitmapFailed(e: java.lang.Exception?, errorDrawable: Drawable?) {
            Timber.e(e)
        }

        override fun onBitmapLoaded(bitmap: Bitmap?, from: Picasso.LoadedFrom?) {
            Timber.d("bitmap loaded: $bitmap")
            photoView.setImageBitmap(bitmap)
        }
    }

    override fun onViewBound(model: PhotoModel) {

        photoDesc.visibility = if (model.description.isNullOrBlank()) View.GONE else View.VISIBLE
        model.description?.let { photoDesc.text = it }

        (itemView.context as Activity).windowManager.defaultDisplay.getSize(displaySize)

        model.resolutionList.minBy { it.resHeight * it.resWidth }?.let { photoRes ->
            picassoAsync
                .load(photoRes.url)
                .transform(BlurTransformation(itemView.context, 13, 1))
                .transform(ColorFilterTransformation(0xCC000000.toInt()))
                .into(photoView, object : Callback {
                    override fun onSuccess() {
                            model.optimalResolution(displaySize.x, displaySize.y).let {
                                Timber.d("goonna load image with ${it.resWidth} / ${it.resHeight}")
                                Picasso.get()
                                    .load(it.url)
                                    .into(_t)
                            }
                    }
                    override fun onError(e: Exception?) {
                        Timber.e(e)
                    }
                })
        }
    }
}
