package com.biganto.visual.roompark.presentation.screen.web_cam

import android.net.Uri
import android.os.Bundle
import android.view.View
import androidx.core.os.bundleOf
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.google.android.exoplayer2.ExoPlayerFactory
import com.google.android.exoplayer2.SimpleExoPlayer
import com.google.android.exoplayer2.ext.rtmp.RtmpDataSourceFactory
import com.google.android.exoplayer2.extractor.ts.DefaultTsPayloadReaderFactory
import com.google.android.exoplayer2.source.MediaSource
import com.google.android.exoplayer2.source.ProgressiveMediaSource
import com.google.android.exoplayer2.source.hls.DefaultHlsDataSourceFactory
import com.google.android.exoplayer2.source.hls.DefaultHlsExtractorFactory
import com.google.android.exoplayer2.source.hls.HlsMediaSource
import com.google.android.exoplayer2.trackselection.AdaptiveTrackSelection
import com.google.android.exoplayer2.trackselection.DefaultTrackSelector
import com.google.android.exoplayer2.ui.PlayerView
import com.google.android.exoplayer2.upstream.DefaultBandwidthMeter
import com.google.android.exoplayer2.upstream.DefaultDataSourceFactory
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_CAM_INDEX_KEY = "SELECTED_CAM_INDEX"

class WebCamScreenController :
    BigantoBaseController<WebCamScreenViewState
            , WebCamScreen
            , WebCamScreenPresenter>
   , WebCamScreen {

    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_CAM_INDEX_KEY to id))

    @BindView(R.id.webCamPlayerView)
    lateinit var playerView:PlayerView

    lateinit var player:SimpleExoPlayer


    override fun onViewBound(v: View) {
        toolBar.setToolbar()
        bottomNavigationController.hide()
    }

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: WebCamScreenPresenter

    fun getComponent() = DaggerWebCamScreenComponent
        .factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_CAM_INDEX_KEY))
        .inject(this)

    override fun render(viewState: WebCamScreenViewState) {
        when(viewState){
            is WebCamScreenViewState.Idle -> render(viewState)
            is WebCamScreenViewState.CamSelected -> render(viewState)
            is WebCamScreenViewState.WebCamsLIstLoaded-> render(viewState)
            is WebCamScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: WebCamScreenViewState.Idle){

    }

    private fun render(viewState: WebCamScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: WebCamScreenViewState.CamSelected){

    }

    private fun render(viewState: WebCamScreenViewState.WebCamsLIstLoaded){

        val bandwidthMeter = DefaultBandwidthMeter.Builder(activity).build()
        val videoTrackSelectionFactory = AdaptiveTrackSelection.Factory()
        val trackSelector = DefaultTrackSelector(videoTrackSelectionFactory)

        player = ExoPlayerFactory.newSimpleInstance(activity!!,
            trackSelector
        )

//        player.addListener(object : Player.EventListener {
//
//            override fun onTimelineChanged(timeline: Timeline?, manifest: Any?, reason: Int) {
//                Timber.d("onTimelineChanged: ");
//            }
//
//            override fun onTracksChanged(
//                trackGroups: TrackGroupArray?,
//                trackSelections: TrackSelectionArray?
//            ) {
//                Timber.d("onTracksChanged: ");
//            }
//
//            override fun onLoadingChanged(isLoading: Boolean) {
//                Timber.d("onLoadingChanged: ");
//            }
//
//            override fun onPlayerStateChanged(playWhenReady: Boolean, playbackState: Int) {
//                Timber.d("onPlayerStateChanged: $playWhenReady");
//                Timber.d("onPlayerStateChanged: $playbackState")
//                Timber.d("onPlayerStateChanged: ${player.isPlaying}")
//
//
//            }
//
//            override fun onRepeatModeChanged(repeatMode: Int) {
//                Timber.d("onRepeatModeChanged: ");
//            }
//
//            override fun onShuffleModeEnabledChanged(shuffleModeEnabled: Boolean) {
//                Timber.d("onShuffleModeEnabledChanged");
//            }
//
//            override fun onPlayerError(error: ExoPlaybackException?) {
//                Timber.d("onPlayerError")
//            }
//
//            override fun onPositionDiscontinuity(reason: Int) {
//                Timber.d("onPositionDiscontinuity")
//            }
//
//            override fun onPlaybackParametersChanged(playbackParameters: PlaybackParameters?) {
//                Timber.d("onPlaybackParametersChanged")
//            }
//
//            override fun onPlaybackSuppressionReasonChanged(playbackSuppressionReason: Int) {
//                Timber.d("onPlaybackSuppressionReasonChanged")
//            }
//
//            override fun onIsPlayingChanged(isPlaying: Boolean) {
//                Timber.d("onIsPlayingChanged")
//            }
//
//        })

        playerView.player = player

        val sourceUrl = viewState.camList.first{it.index == viewState.selectedCamIndex}.streams
            .first().hls
//        var rtmpUrl = viewState.camList.first{it.index == viewState.selectedCamIndex}.streams
//            .first().rtmp
//        rtmpUrl = "rtmp://room-park.ru:1935/cam2-360/stream"
//        Timber.d("source url : ${Uri.parse(rtmpUrl)}")
        val mediaSource = buildMediaSource(Uri.parse(sourceUrl))
//        val mediaSource = buildRtmps(Uri.parse(rtmpUrl))
        Timber.d("source url : ${Uri.parse(sourceUrl)}")
        Timber.d("mediaSource : $mediaSource")
        player.prepare(mediaSource,true,true)
        player.playWhenReady = true
        Timber.d(" after all: ${player.playbackState}")
//        player.seekToDefaultPosition()

    }


    override fun getLayoutId(): Int = R.layout.webcam_screen

    private fun buildMediaSource(uri: Uri): MediaSource {
        val source = DefaultDataSourceFactory(activity,"ua")
        val dataSourceFactory = DefaultHlsDataSourceFactory(source)//.createDataSource(DATA_TYPE_DRM)
        val extr = DefaultHlsExtractorFactory(DefaultTsPayloadReaderFactory.FLAG_IGNORE_AAC_STREAM,true)

        (DefaultTsPayloadReaderFactory.FLAG_ALLOW_NON_IDR_KEYFRAMES)
        return HlsMediaSource.Factory(dataSourceFactory)
            .setExtractorFactory(extr)
            .setAllowChunklessPreparation(true)
            .createMediaSource(uri)
    }

    private fun buildRtmps(uri: Uri): MediaSource {
        val source = DefaultDataSourceFactory(activity,"ua")
        val dataSourceFactory = RtmpDataSourceFactory()
        return ProgressiveMediaSource.Factory(dataSourceFactory)
            .createMediaSource(uri)
    }

}